import { defineStore } from "pinia";
import { useConnectionStore } from "./connection";
import * as Y from "yjs";
import { Awareness } from "y-protocols/awareness.js";
import { computed, ref } from "vue";

export const useMissionDetailStore = defineStore(
  "missionDetail",
  () => {
    const connectionStore = useConnectionStore();

    const ydoc = ref(new Y.Doc());
    const awareness = ref(new Awareness(ydoc.value));

    const editor = ref(ydoc.value.getText("editor"));

    const title = computed({
      get() {
        return ydoc.value.getMap("form").get("title") ?? "";
      },
      set(val) {
        ydoc.value.getMap("form").set("title", val);
      },
    });

    function init(missionId: string) {
      connectionStore.connection?.emit("room:join", missionId);

      connectionStore.connection?.on("sync", (update) => {
        Y.applyUpdate(ydoc.value, update);
      });
    }
    function destroy() {
      connectionStore?.connection?.emit("room:leave");
      useMissionDetailStore().$reset();
    }

    ydoc.value.on("update", (update) => {
      console.log(Array.from(update));
      connectionStore.connection?.emit("sync", Array.from(update));

      ydoc.value = new Y.Doc();
      awareness.value = new Awareness(ydoc.value);
      editor.value = ydoc.value.getText("editor");
    });

    return { ydoc, awareness, editor, title, init, destroy };
  }
  // {
  //   state: () => {
  //     const ydoc = new Y.Doc();
  //     return {
  //       ydoc: ydoc as Y.Doc,
  //       awareness: new Awareness(ydoc) as Awareness,

  //       editor: "",
  //     };
  //   },
  //   actions: {
  //     init(missionId: string) {
  //       const connectionStore = useConnectionStore();
  //       if (!connectionStore.connection) return;

  //       connectionStore.connection.emit("room:join", missionId);

  //       connectionStore.connection.on("sync", (update) => {
  //         Y.applyUpdate(this.ydoc, update);
  //       });

  //       this.$subscribe((mutation, state) => {
  //         console.log(mutation, state);
  //       });
  //     },
  //     destroy() {
  //       const connectionStore = useConnectionStore();
  //       if (!connectionStore.connection) return;
  //       connectionStore.connection.emit("room:leave");
  //       this.$reset();
  //     },
  //   },
  // }
);
